//------------------------------------------------------------------------------
//
//  npctrl.c
//
//  ANTz application based on NPE is free software
//	NPE is the Neural Physics Engine that drives the interactions
//
//	ANTz hosted at http://temporalzone.com and NPE at http://neuralphysics.org
//
//	Copyright (c) 2010-2011, Shane Saxon
//	All rights reserved.
//	Redistribution and use in source and binary forms, with or without 
//	modification, are permitted provided that the following conditions are met:
//		Redistributions of source code must retain the above copyright notice, 
//		this list of conditions and the following disclaimer.
//		Redistributions in binary form must reproduce the above copyright 
//		notice, this list of conditions and the following disclaimer in the 
//		documentation and/or other materials provided with the distribution.
//		Neither the name of the Shane Saxon nor the names of its contributors 
//		may be used to endorse or promote products derived from this software 
//		without specific prior written permission.
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
//	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//	POSSIBILITY OF SUCH DAMAGE.
//
//------------------------------------------------------------------------------

#include "npdata.h"
#include "npio.h"
#include "npctrl.h"

#include "data/npnode.h"
#include "io/npgl.h"
#include "io/npfile.h"
#include "ctrl/npengine.h"

void npCtrlFile (int command, void* dataRef);

void npCtrlVector (int command, void* dataRef);
void npCtrlSelect (int command, void* dataRef);
void npCtrlProperty (int command, void* dataRef);
void npCtrlGlobal (int command, void* dataRef);

void TraverseTree (int command, NPnodePtr node, void* dataRef);

//------------------------------------------------------------------------------
void npInitCtrl (void* dataRef)
{
	npInitEngine (dataRef);
}

//------------------------------------------------------------------------------
void npCloseCtrl (void* dataRef)
{
	npCloseEngine (dataRef);
}

//------------------------------------------------------------------------------
int npAppLoop (void* dataRef)
{
	int err = 0;

#ifdef NP_LINUX_
//	MainAppLoopLinux();
#endif

#ifdef NP_MSW_
//	MainAppLoopMSW();
	npAppLoopGlut (dataRef);
#endif

#ifdef NP_OSX_
	//	MainAppLoopOSX();
#endif


	return err;
}



//------------------------------------------------------------------------------
void TraverseTree (int command, NPnodePtr node, void* dataRef)
{
	int i = 0;
	
	pData data = (pData) dataRef;

	// recursively iterate through all nodes in the tree
	for (i=0; i < node->childCount ; i++)
		TraverseTree (command, node->child[i], dataRef);
	
	if ( node->selected 
		 || command == kGraphSelectOn 
		 || command == kGraphSelectOff )
	{
		data->map.activeNode = node;
		npCtrlCommandSet (command, node, dataRef);
	}
}


//------------------------------------------------------------------------------
void npCtrlCommand (int command, void* dataRef)
{
	int i = 0;
	int indexTemp = 0;

	pData data = (pData) dataRef;

	NPnodePtr node = data->map.activeNode;
	NPnodePtr nodeParent = NULL;


	switch (command)
	{
		// kCmdGlobal... global commands, draw indicators/HUD, quit
//		case kQuit : npCtrlGlobal (command, dataRef); break;

		case kFileNew : npCtrlFile (command, dataRef); break;
		case kFileOpen : npCtrlFile (command, dataRef); break;
		case kFileClose : npCtrlFile (command, dataRef); break;
		case kFileSave : npCtrlFile (command, dataRef); break;
		case kFileSaveAs : npCtrlFile (command, dataRef); break;
		case kFileImport : npCtrlFile (command, dataRef); break;
		case kFileExport : npCtrlFile (command, dataRef); break;

		case kGraphMenu : npCtrlGlobal (command, dataRef); break;
		case kFullScreenExit : npCtrlGlobal (command, dataRef); break;
		case kGraphClearFrameBuffer : npCtrlGlobal (command, dataRef); break;
		case kGraphBackground : npCtrlGlobal (command, dataRef); break;		
		case kGraphAlphaMode : npCtrlGlobal (command, dataRef); break;

		// kCmdSelect.. modify selection / scope (navigate the map/space)	
		// Selects the active axes and nodes to manipulate
		case kGraphSelectAxes : npCtrlSelect (command, dataRef); break;

		case kGraphGrid : npCtrlSelect (command, dataRef); break;
		case kGraphCamera : npCtrlSelect (command, dataRef); break;

		case kGraphNext : npCtrlSelect (command, dataRef); break;
		case kGraphNextBranch : npCtrlSelect (command, dataRef); break;

		case kGraphNew : npCtrlSelect (command, dataRef); break;
		case kGraphDelete : npCtrlSelect (command, dataRef); break;

//		case kGraphSelect : npCtrlSelect (command, dataRef); break;	// broken, debug zz
//		case kGraphSelectOn : npCtrlSelect (command, dataRef); break;	// broken, debug zz
//		case kGraphSelectOff : npCtrlSelect (command, dataRef); break;	// broken, debug zz

		case kGraphSelectAll : npCtrlSelect (command, dataRef); break;		// broken, debug zz
		case kGraphSelectAllToggle : npCtrlSelect (command, dataRef); break;
		case kGraphSelectNone : npCtrlSelect (command, dataRef); break;
		case kGraphSelectOne : npCtrlSelect (command, dataRef); break;
		case kGraphSelectTwo : npCtrlSelect (command, dataRef); break;
		case kGraphSelectThree : npCtrlSelect (command, dataRef); break;

		case kGraphPresetOne : npDataPreset (command, dataRef); break;
		case kGraphPresetTwo : npDataPreset (command, dataRef); break;
		case kGraphPresetThree : npDataPreset (command, dataRef); break;
		case kGraphPresetFour : npDataPreset (command, dataRef); break;
		case kGraphPresetFive : npDataPreset (command, dataRef); break;
		case kGraphPresetSix : npDataPreset (command, dataRef); break;
		case kGraphPresetSeven : npDataPreset (command, dataRef); break;
		case kGraphPresetEight : npDataPreset (command, dataRef); break;
		

		default : // not a global command

			// exclude global commands or only 1 selected node from applying to all nodes-
			// ie. allows for adding an unsellected node to the current group
			if ( !node->selected || data->map.nodeRootIndex < 3 )
			{
				node = data->map.node[data->map.nodeRootIndex];
				npCtrlCommandSet (command, node, data);
			}
			else
			{											// apply to selected nodes
				indexTemp = data->map.nodeRootIndex;
	
				if (node->branchLevel > 0)
					nodeParent = node->parent;
	
				for (i=0; i < data->map.nodeRootCount; i++)	//search and apply to selected nodes
				{
					data->map.nodeRootIndex = i;
					TraverseTree (command, data->map.node[i], dataRef);
				}
					data->map.nodeRootIndex = indexTemp;
					data->map.activeNode = node;
			}	
		break;
	}
}


//------------------------------------------------------------------------------
void npCtrlCommandSet (int command, void* nodeRef, void* dataRef)
{
	pData data = (pData) dataRef;

	NPnodePtr node = (NPnodePtr) nodeRef;


	switch (command)
	{
		case kGraphSelect : npCtrlSelect (command, dataRef); break;	// broken, debug zz
		case kGraphSelectOn : npCtrlSelect (command, dataRef); break;	// broken, debug zz
		case kGraphSelectOff : npCtrlSelect (command, dataRef); break;	// broken, debug zz

//		case kGraphSelectAll : npCtrlSelect (command, dataRef); break;		// broken, debug zz

		//translate the selected objects relative to the camera
		// kCmd... apply to current selection / scope (X/Y/XY/Z/XYZ)
		case kGraphXdecrease : npCtrlVector (command, dataRef); break;
		case kGraphXdecreaseOff : npCtrlVector (command, dataRef); break;
		case kGraphXincrease : npCtrlVector (command, dataRef); break;
		case kGraphXincreaseOff : npCtrlVector (command, dataRef); break;

		case kGraphYdecrease : npCtrlVector (command, dataRef); break;
		case kGraphYdecreaseOff : npCtrlVector (command, dataRef); break;
		case kGraphYincrease : npCtrlVector (command, dataRef); break;
		case kGraphYincreaseOff : npCtrlVector (command, dataRef); break;

		case kGraphZdecrease : npCtrlVector (command, dataRef); break;
		case kGraphZdecreaseOff : npCtrlVector (command, dataRef); break;
		case kGraphZincrease : npCtrlVector (command, dataRef); break;
		case kGraphZincreaseOff : npCtrlVector (command, dataRef); break;

		case kGraphRotateLeft : npCtrlVector (command, dataRef); break;
		case kGraphRotateLeftOff : npCtrlVector (command, dataRef); break;
		case kGraphRotateRight : npCtrlVector (command, dataRef); break;
		case kGraphRotateRightOff : npCtrlVector (command, dataRef); break;

		case kGraphRotateUp : npCtrlVector (command, dataRef); break;
		case kGraphRotateUpOff : npCtrlVector (command, dataRef); break;
		case kGraphRotateDown : npCtrlVector (command, dataRef); break;
		case kGraphRotateDownOff : npCtrlVector (command, dataRef); break;

		case kGraphRotateCCW : npCtrlVector (command, dataRef); break;
		case kGraphRotateCCWOff : npCtrlVector (command, dataRef); break;
		case kGraphRotateCW : npCtrlVector (command, dataRef); break;
		case kGraphRotateCWOff : npCtrlVector (command, dataRef); break;

		case kGraphZoomOn : npCtrlVector (command, dataRef); break;
		case kGraphZoomOff : npCtrlVector (command, dataRef); break;

		// Change node property
		case kGraphChannelUp : npCtrlProperty (command, dataRef); break;
		case kGraphChannelDown : npCtrlProperty (command, dataRef); break;

		case kGraphColorUp : npCtrlProperty (command, dataRef); break;
		case kGraphColorDown : npCtrlProperty (command, dataRef); break;
		case kGraphAlphaUp : npCtrlProperty (command, dataRef); break;
		case kGraphAlphaDown : npCtrlProperty (command, dataRef); break;
					
		case kGraphCenter : npCtrlProperty (command, dataRef); break;
		case kGraphLines : npCtrlProperty (command, dataRef); break;
		case kGraphPoints : npCtrlProperty (command, dataRef); break;
		case kGraphSurface : npCtrlProperty (command, dataRef); break;
		case kGraphPrimitive : npCtrlProperty (command, dataRef); break;
		case kGraphMesh : npCtrlProperty (command, dataRef); break;
		case kGraphScroll : npCtrlProperty (command, dataRef); break;
		case kGraphFreeze : npCtrlProperty (command, dataRef); break;
		case kGraphHide : npCtrlProperty (command, dataRef); break;
		case kGraphClear : npCtrlProperty (command, dataRef); break;

		case kGraphDrawIndicators : npCtrlProperty (command, dataRef); break;

		case kGraphSampleInterval : npCtrlProperty (command, dataRef); break;

		case kEventLowSetpoint : npCtrlProperty (command, dataRef); break;
		case kEventHighSetpoint : npCtrlProperty (command, dataRef); break;

		default : break;
	}
}


//------------------------------------------------------------------------------
void npCtrlFile (int command, void* dataRef)
{
	pData data = (pData) dataRef;

	switch (command)
	{
		case kFileNew : npFileDialog (NULL, kNPfileDialogNew, dataRef); break;

		case kFileOpen : npSetMapFile ("ANTZ0001.CSV", 1, 13, dataRef); break; //npFileDialog (NULL, kNPfileDialogOpen, dataRef); break;
		case kFileClose : npFileDialog (NULL, kNPfileDialogClose, dataRef); break;

		case kFileSave : npTestMapFile  ("ANTZ0001.CSV", 1, 13, dataRef); break; //npFileDialog (NULL, kNPfileDialogSave, dataRef); break;
		case kFileSaveAs : npFileDialog (NULL, kNPfileDialogSaveAs, dataRef); break;

		case kFileImport : npSetChannelFile ("TNG0001.CSV", 1, 13, dataRef); break; //npFileDialog (NULL, kNPfileDialogImport, dataRef); break;
		case kFileExport : npFileDialog (NULL, kNPfileDialogExport, dataRef); break;

		default : break;
	}
}


// update the translateVect for selected objects
// translateVect is relative to global coordinates
// translateRate is relative to the camera
//------------------------------------------------------------------------------
void npCtrlVector (int command, void* dataRef)
{
	float velocity = 0.0f;

	NPnodePtr node = NULL;

	pData data = (pData) dataRef;

	// apply command to activeNode
	node = data->map.activeNode;

	// if shift pressed then use fast velocity, otherwise slow
	if (data->io.key.modShift)
		velocity = data->ctrl.fast;
	else
		velocity = data->ctrl.slow;

	switch (command)
	{								//normalize and change to 0.0f - velocity,	debug zz
		case kGraphXdecrease : node->translateRate.x = -0.2f * velocity; break;
		case kGraphXdecreaseOff : 
			if (node->translateRate.x < 0.0f)	//graceful multi-key behavior
				node->translateRate.x = 0.0f; 
			break;
		case kGraphXincrease : node->translateRate.x = 0.2f * velocity; break;
		case kGraphXincreaseOff : 
			if (node->translateRate.x > 0.0f)
				node->translateRate.x = 0.0f; 
			break;

		case kGraphYdecrease : node->translateRate.y = -0.2f * velocity; break;
		case kGraphYdecreaseOff : 
			if (node->translateRate.y < 0.0f)
				node->translateRate.y = 0.0f;
			break;
		case kGraphYincrease : node->translateRate.y = 0.2f * velocity; break;
		case kGraphYincreaseOff : 
			if (node->translateRate.y > 0.0f)
				node->translateRate.y = 0.0f;
			break;

		case kGraphZdecrease : node->translateRate.z = -0.2f * velocity; break;
		case kGraphZdecreaseOff : 
			if (node->translateRate.z < 0.0f)
				node->translateRate.z = 0.0f;
			break;
		case kGraphZincrease : node->translateRate.z = 0.2f * velocity; break;
		case kGraphZincreaseOff : 
			if (node->translateRate.z > 0.0f)
				node->translateRate.z = 0.0f;
			break;
		
		case kGraphRotateLeft : node->rotateRate.z = -0.005f * velocity; break;
		case kGraphRotateLeftOff : 
			if (node->rotateRate.z < 0.0f)
				node->rotateRate.z = 0.0f;
			break;
		case kGraphRotateRight : node->rotateRate.z = 0.005f * velocity; break;
		case kGraphRotateRightOff : 
			if (node->rotateRate.z > 0.0f)
				node->rotateRate.z = 0.0f;
			break;

		case kGraphRotateUp : node->rotateRate.x = 0.005f * velocity; break;
		case kGraphRotateUpOff :
			if (node->rotateRate.x > 0.0f)
				node->rotateRate.x = 0.0f;
			break;
		case kGraphRotateDown : node->rotateRate.x = -0.005f * velocity; break;
		case kGraphRotateDownOff :
			if (node->rotateRate.x < 0.0f)
				node->rotateRate.x = 0.0f;
			break;

		case kGraphRotateCCW : node->rotateRate.y = -0.005f * velocity; break;
		case kGraphRotateCCWOff : 
			if (node->rotateRate.y < 0.0f)
				node->rotateRate.y = 0.0f;
			break;
		case kGraphRotateCW : node->rotateRate.y = 0.005f * velocity; break;
		case kGraphRotateCWOff : 
			if (node->rotateRate.y > 0.0f)
				node->rotateRate.y = 0.0f;
			break;

		case kGraphZoomOn :
			if (data->io.key.modShift)
			{
				if (data->io.axes.x == true )
					node->scaleRate.x = -0.003f;
				if (data->io.axes.y == true )
					node->scaleRate.y = -0.003f;
				if (data->io.axes.z == true )
					node->scaleRate.z = -0.003f;			
			}
			else
			{
				if (data->io.axes.x == true )
					node->scaleRate.x = 0.003f;
				if (data->io.axes.y == true )
					node->scaleRate.y = 0.003f;
				if (data->io.axes.z == true )
					node->scaleRate.z = 0.003f;			
			}
			break;
		case kGraphZoomOff :
			node->scaleRate.x = 0.0f;
			node->scaleRate.y = 0.0f; 
			node->scaleRate.z = 0.0f; 
			break;

		default : break;
	}
}

//------------------------------------------------------------------------------
void npCtrlSelect (int command, void* dataRef)
{
	int i = 0;
	int commandTemp = 0;

	NPnodePtr node = NULL;
	NPnodePtr nodeChild = NULL;

	pData data = (pData) dataRef;

	node = data->map.activeNode;


	switch (command)
	{							// add ability to copy existing node parameters, debug zz
		case kGraphNew : 
			if (node->branchLevel > 0 && data->map.nodeRootIndex >= kNPpinRoot)
				npNodeNew (node->type, node, dataRef); 
			else
				npNodeNew (kNodePin, NULL, dataRef);
			break;

		case kGraphDelete :
			if (data->map.nodeRootIndex < kNPpinRoot)		//zz debug, does not support grid and cam
				return;
			npNodeDelete (node, dataRef); break;
			
		case kGraphNext :
			if (data->map.nodeRootCount <= kNPpinRoot)		// do nothing if no nodes
				return;
			if (data->map.nodeRootIndex < kNPpinRoot)		// camera or grid selected
			{
				//restore the previously selected node
				data->map.nodeRootIndex = data->map.selectedIndex;
				data->map.activeNode = data->map.selectedNode;
			}
			else
			{
				if (node->branchLevel < 1)				// root level pin
				{
					// change our root node selection
					if  (data->io.key.modShift)
						data->map.nodeRootIndex--;
					else
						data->map.nodeRootIndex++;

					// keep inbounds and don't select camera or grid
					if (data->map.nodeRootIndex >= data->map.nodeRootCount)
							data->map.nodeRootIndex = kNPpinRoot;
					else if (data->map.nodeRootIndex < kNPpinRoot)	// select last pin
						data->map.nodeRootIndex = data->map.nodeRootCount - 1;

					// select our next root node and make active
					data->map.selectedIndex = data->map.nodeRootIndex;
					data->map.selectedNode = data->map.node[data->map.selectedIndex];
					data->map.activeNode = data->map.selectedNode;
				}
				else
				{						// if available, select next sibling
					nodeChild = node;
					node = nodeChild->parent;

					if  (data->io.key.modShift)
						node->childIndex--;
					else
						node->childIndex++;
		
					// keep in bounds and roll over between first and last
					if (node->childIndex >= node->childCount)
						node->childIndex = 0;
					else if (node->childIndex < 0)
						node->childIndex = node->childCount - 1;

					// select our next sibling node and make active
					data->map.selectedNode = node->child[node->childIndex];
					data->map.activeNode = data->map.selectedNode;
				}
			}
			break;

		case kGraphNextBranch : 
			if (data->map.nodeRootIndex < kNPpinRoot)		//zz debug, does not support grid and cam
				return;
			if (data->io.key.modShift)
			{
				if (node->branchLevel > 0)		// do nothing if at the trunk
				{
					// traverse up 1 level to the childs parent
					data->map.selectedNode = node->parent;
					data->map.activeNode = data->map.selectedNode;
				}
			}		// traverse down 1 level if selected child exists
			else if (node->childCount > node->childIndex)
			{
				data->map.selectedNode = node->child[node->childIndex];
				data->map.activeNode = data->map.selectedNode;
			}
			break;
				
		case kGraphGrid :
			data->map.nodeRootIndex = kNPgridRoot;		// select grid
			data->map.activeNode = data->map.node[data->map.nodeRootIndex];
			break;

		case kGraphCamera :
			node = data->map.node[kNPcameraRoot];
			if (data->map.nodeRootIndex == kNPcameraRoot)	// camera selected
			{
				// reset the camera if shift held, else change cameras
				if (data->io.key.modShift)
					npDataCameraPreset (node->childIndex, data->map.activeNode, data);
				else
				{
					// select next camera
					node->childIndex++;
					if (node->childIndex >= node->childCount)
						node->childIndex = 0;
					data->map.activeNode = node->child[node->childIndex];
				}
			}
			else	// select the current camera
			{
				data->map.nodeRootIndex = kNPcameraRoot;
				data->map.activeNode = node->child[node->childIndex];
				// if alt pressed, reset camera
				if (data->io.key.modShift)
					npDataCameraPreset(node->childIndex, data->map.activeNode, data);
			}
			break;

		case kGraphSelectAxes :
			if (data->io.axes.x == true 
				&& data->io.axes.y == true
				&& data->io.axes.z == true )
			{
				data->io.axes.z = false;
				printf ("\n axes: XY");				// make part of HUD debug, zz
			}
			else if (data->io.axes.x == true 
					 && data->io.axes.y == true
					 && data->io.axes.z == false )
			{
				data->io.axes.x = true; 
				data->io.axes.y = false; 
				data->io.axes.z = false;
				printf ("\n axes: X");
			}
			else if (data->io.axes.x == true 
					 && data->io.axes.y == false
					 && data->io.axes.z == false )
			{
				data->io.axes.x = false; 
				data->io.axes.y = true; 
				data->io.axes.z = false;
				printf ("\n axes: Y");
			}
			else if (data->io.axes.x == false 
					 && data->io.axes.y == true
					 && data->io.axes.z == false )
			{
				data->io.axes.x = false; 
				data->io.axes.y = false; 
				data->io.axes.z = true;
				printf ("\n axes: Z");
			}
			else if (data->io.axes.x == false 
					 && data->io.axes.y == false
					 && data->io.axes.z == true )
			{
				data->io.axes.x = true; 
				data->io.axes.y = true; 
				data->io.axes.z = true;
				printf ("\n axes: XYZ");
			}
			break;

		case kGraphSelect :
			if (data->map.nodeRootIndex < kNPpinRoot)	//zz debug, does not support cam and grid
				return;
			if (node->selected)
				node->selected = false;
			else
				node->selected = true;
			break;
		case kGraphSelectOn :
			node->selected = true;
			break;
		case kGraphSelectOff :
			node->selected = false;
			break;

		case kGraphSelectAll :
			if (data->map.nodeRootIndex < kNPpinRoot)		// if cam or grid then ...
				npCtrlCommand (kGraphNext, data);		// select our previous node
			if (data->map.selectAll)
			{
				data->map.selectAll = false;
				commandTemp = kGraphSelectOff;
			}
			else
			{
				data->map.selectAll = true;
				commandTemp = kGraphSelectOn;
			}
			
			data->map.selectedNode = data->map.activeNode;		//zz debug, should not be necessary
			
			for (i=kNPpinRoot; i < data->map.nodeRootCount; i++)
				TraverseTree (commandTemp, data->map.node[i], data);
			
			data->map.activeNode = data->map.selectedNode;
			break;
		case kGraphSelectAllToggle :						//zz flip with SelectAll, debug
			if (data->map.nodeRootIndex < kNPpinRoot)			// if cam or grid then ...
				npCtrlCommand (kGraphNext, data);		// select our previous node
	
			data->map.selectAll = false;
			commandTemp = kGraphSelectOff;
			
			data->map.selectedNode = data->map.activeNode;
			
			for (i=kNPpinRoot; i < data->map.nodeRootCount; i++)
				TraverseTree (commandTemp, data->map.node[i], data);
			
			data->map.activeNode = data->map.selectedNode;
			break;
		case kGraphSelectNone :
			data->map.selectionSet.x = false;
			data->map.selectionSet.y = false;
			data->map.selectionSet.z = false;
			break;
		case kGraphSelectOne :
//			data->map.selectionSet.x = 1 - data->map.selectionSet.x;	//zz debug
			if (data->io.key.modShift)
				npTestMapFile  ("ANTZ0001.CSV", 1, 13, dataRef);
			else
				npSetMapFile ("ANTZ0001.CSV", 1, 13, dataRef);
			break;
		case kGraphSelectTwo :
//			data->map.selectionSet.y = 1 - data->map.selectionSet.y;
			if (data->io.key.modShift)
				npTestMapFile  ("ANTZ0002.CSV", 1, 13, dataRef);
			else
				npSetMapFile ("ANTZ0002.CSV", 1, 13, dataRef);
			break;
		case kGraphSelectThree :
//			data->map.selectionSet.z = 1 - data->map.selectionSet.z;
			if (data->io.key.modShift)
				npTestMapFile  ("ANTZ0003.CSV", 1, 13, dataRef);
			else
				npSetMapFile ("ANTZ0003.CSV", 1, 13, dataRef);			//zz end
			break;


		default : break;
	}
}


//------------------------------------------------------------------------------
void npCtrlProperty (int command, void* dataRef)
{
	pData data = (pData) dataRef;

	NPnodePtr node = NULL;
	NPgridPtr grid = NULL;	

	if (data->map.nodeRootIndex < 3)
		node = data->map.node[data->map.nodeRootIndex];
	else
		node = data->map.activeNode;


	switch (command)
	{
		case kGraphChannelDown :

			if (data->io.axes.x)
			{
				node->channel.x++; 
				if (node->channel.x >= data->io.channelCount)
					node->channel.x = 0;
			}
			if (data->io.axes.y)
			{
				node->channel.y++; 
				if (node->channel.y >= data->io.channelCount)
					node->channel.y = 0;
			}
			if (data->io.axes.z)
			{
				node->channel.z++; 
				if (node->channel.z >= data->io.channelCount)
					node->channel.z = 0;
			}
			printf ("\n X: %d   Y: %d   Z: %d ", 
					node->channel.x, node->channel.y, node->channel.z);
			break;
		case kGraphChannelUp :
			if (data->io.axes.x)
			{
				node->channel.x--; 
				if (node->channel.x < 0)
					node->channel.x = data->io.channelCount - 1;
			}
			if (data->io.axes.y)
			{
				node->channel.y--; 
				if (node->channel.y < 0)
					node->channel.y = data->io.channelCount - 1;
			}
			if (data->io.axes.z)
			{
				node->channel.z--; 
				if (node->channel.z < 0)
					node->channel.z = data->io.channelCount - 1;
			}
			printf ("\n X: %d   Y: %d   Z: %d ", 
					node->channel.x, node->channel.y, node->channel.z);
			break;

//		case kGraphSampleInterval : ; break;

		case kGraphColorUp :
			node->colorIndex++;
			SetIndexColor (&node->color, &node->colorIndex);
			break;
		case kGraphColorDown :
			node->colorIndex--;
			SetIndexColor (&node->color, &node->colorIndex);
			break;

		case kGraphAlphaUp :
			if (node->color.a < 246)
				node->color.a += 10;
			break;
		case kGraphAlphaDown :
			if (node->color.a > 9)
				node->color.a -= 10;
			break;

		case kGraphCenter : node->center = 1 - node->center; break;

		case kGraphLines :
			node->lineWidth += 1.0f;
			if (node->lineWidth > 5.0f)
				node->lineWidth = 0.0f;
			break;
		case kGraphPoints :
			node->pointSize += 1.0f;
			if (node->pointSize > 5.0f)
				node->pointSize = 0;
			break;
		case kGraphSurface :
			if (data->io.key.modShift)
			{
				node->surfaceType--;
				if (node->surfaceType < 0)
					node->surfaceType = 16 - 1;								//zz debug
				//	node->surfaceType = kSurfaceTypeCount - 1;
			}
			else
			{
				node->surfaceType++;
				if (node->surfaceType >= 16)
					node->surfaceType = 0;
				//	node->surfaceType = 0;
			}
			printf("\n %d", node->surfaceType);
			break;
		case kGraphPrimitive :
			if (data->io.key.modShift)
			{
				node->shaderType--;
				if (node->shaderType < 0)
					node->shaderType = kNPglutCount - 3;	//zz -3 instead -1 to remove teapot, debug
			}
			else
			{
				node->shaderType++;
				if (node->shaderType >= kNPglutCount - 2)
					node->shaderType = 0;
			}
			printf("\n %d", node->surfaceType);
			break;
		case kGraphMesh : node->surfaceType = kSurfaceMesh; break;
		case kGraphScroll : node->scroll = 1 - node->scroll; break;
		case kGraphFreeze : node->freeze = 1 - node->freeze; break;
		case kGraphHide : node->hide = 1 - node->hide; break;

		case kGraphClear :
			node->triggerOnLow.x = false;
			node->triggerOnLow.y = false;
			node->triggerOnLow.z = false;
			node->triggerOnHi.x = false;
			node->triggerOnHi.y = false;
			node->triggerOnHi.z = false;
			break;

		case kGraphSampleInterval : 
			if (node->type != kNodeGrid)
			{
/*				if (data->io.key.modShift)
					node->sampleInterval /= 2;
				else
					node->sampleInterval *= 2;
				if (node->sampleInterval < 1)
					node->sampleInterval = 1;
*/			}
			else
			{
				grid = (NPgridPtr) node->data;

				if (data->io.key.modShift)
				{
					if (data->io.axes.x)
					{
						grid->segments.x--;
						if (grid->segments.x < 0)
							grid->segments.x = 0;
					}
					if (data->io.axes.y)
					{
						grid->segments.y--;
						if (grid->segments.y < 0)
							grid->segments.y = 0;
					}
					if (data->io.axes.z)
					{
						grid->segments.z--;
						if (grid->segments.z < 0)
							grid->segments.z = 0;
					}
				}
				else
				{
					if (data->io.axes.x)
						grid->segments.x++;
					if (data->io.axes.y)
						grid->segments.y++;
					if (data->io.axes.z)
						grid->segments.z++;
				}
			}
			break;

		case kEventLowSetpoint :
			if (data->io.axes.x)
			{
				node->triggerLevelLow.x = node->translate.x;
				node->triggerOnLow.x = 1 - node->triggerOnLow.x;
			}
			if (data->io.axes.y)
			{
				node->triggerLevelLow.y = node->translate.y;
				node->triggerOnLow.y = 1 - node->triggerOnLow.y;
			}
			if (data->io.axes.z)
			{
				node->triggerLevelLow.z = node->translate.z;
				node->triggerOnLow.z = 1 - node->triggerOnLow.z;
			}
			break;
		case kEventHighSetpoint :
			if (data->io.axes.x)
			{
				node->triggerLevelHi.x = node->translate.x;
				node->triggerOnHi.x = 1 - node->triggerOnHi.x;
			}
			if (data->io.axes.y)
			{
				node->triggerLevelHi.y = node->translate.y;
				node->triggerOnHi.y = 1 - node->triggerOnHi.y;
			}
			if (data->io.axes.z)
			{
				node->triggerLevelHi.z = node->translate.z;
				node->triggerOnHi.z = 1 - node->triggerOnHi.z;
			}
			break;

		case kGraphDrawIndicators :
			if (data->io.axes.x)
				node->drawCurrentValue.x;
			if (data->io.axes.y)
				node->drawCurrentValue.y;
			if (data->io.axes.z)
				node->drawCurrentValue.z;
			break;

		default : break;
	}
}


//------------------------------------------------------------------------------
void npCtrlGlobal (int command, void* dataRef)
{
	pData data = (pData) dataRef;

	NPnodePtr node = data->map.node[data->map.nodeRootIndex];

	switch (command)
	{
		case kGraphMenu : data->io.drawMenu = 1 - data->io.drawMenu; break;

		case kFullScreenExit : npGlutFullScreenExit(); break;

		case kGraphClearFrameBuffer :
			data->io.clearFrameBuffer = 1 - data->io.clearFrameBuffer;
			break;

		case kGraphBackground :
			if (data->io.clearIndex)
			{
				data->io.clear.r = 0.0f;
				data->io.clear.g = 0.0f;
				data->io.clear.b = 0.0f;
				data->io.clear.a = 0.0f;
				
				data->io.clearIndex = 0;
			}
			else
			{
				data->io.clear.r = 1.0f;
				data->io.clear.g = 1.0f;
				data->io.clear.b = 1.0f;
				data->io.clear.a = 1.0f;

				data->io.clearIndex = 1;
			}
			break;

		case kGraphAlphaMode :
				data->io.alphaMode++;
				if (data->io.alphaMode >= kNPalphaModeCount)
					data->io.alphaMode = 0;
				printf (" %d ", data->io.alphaMode);
			break;

		default : break;
	}
}


//------------------------------------------------------------------------------
void npUpdateCtrl (void* dataRef)
{
	int i = 0;
	static int firstRunLoop = true;

	pData data = (pData) dataRef;

	npUpdateData (dataRef);			//update our data map
	npUpdateIO (dataRef);			//update IO, mouse, sound, txt, video...
	npUpdateEngine (dataRef);		//position, physics, interactions...

	// zero out our mouse to prevent drifting objects
	data->io.mouseDeltaX = 0.0f;											//debug, zz
	data->io.mouseDeltaY = 0.0f;

	// check for command line arguements if first runloop
	if (firstRunLoop)
	{
		firstRunLoop = false;

		// if exist, load command line data files passed in
		for (i=1; i < data->io.argc; i++)		//start at arg 1, 0 is the exe name
		{
			printf("%s\n", data->io.argv[i]);
			npSetMapFile  (data->io.argv[i], 1, 13, dataRef);
		}
	}
}

